<?php

add_action( 'admin_enqueue_scripts', function ( $hook_suffix ) {
	if ( in_array( $hook_suffix, array( 'post.php', 'post-new.php' ) ) ) {
		$screen = get_current_screen();
		if ( is_object( $screen ) && $screen->post_type === "product" ) {
			wp_enqueue_script( 'admin-product-selector', plugins_url( 'admin/js/product-selector.js?v=' . DWD_WOOCOMMERCE_PLUGIN_VERSION, DWD_WOOCOMMERCE_PLUGIN_NAME ) );
			wp_localize_script( 'admin-product-selector', 'dwd_wc', [
			        'DWD_WC_WRAPPER_URL' => DWD_WC_WRAPPER_URL,
                    'DWD_WC_API_HEADERS' => apply_filters('dwd_wc_api_headers', [])
            ]);
		}
	}
} );
add_action( 'woocommerce_product_after_variable_attributes', 'dwd_wc_variable_selector', 10, 3 );
add_filter( 'woocommerce_product_data_tabs', function ( $tabs ) {
	$tabs['dwd-wc-product'] = [
		'label'    => __( 'Drukwerkdeal Product', 'dwd-wc' ),
		'priority' => 50,
		'target'   => 'dwd_wc_view_product_tab',
		'class'    => array( 'show_if_simple' )
	];

	return $tabs;
} );

add_action( 'woocommerce_process_product_meta_simple', 'dwd_wc_save_product' );
add_action( 'woocommerce_process_product_meta_variable', 'dwd_wc_save_product' );

function dwd_wc_save_product( $post_id ) {
	if ( isset( $_POST['dwd_product_uuid'] ) ) {
		update_post_meta( $post_id, 'dwd_product_uuid', $_POST['dwd_product_uuid'][0] );
	}
	if ( isset( $_POST['dwd_product_json'] ) ) {
		update_post_meta( $post_id, 'dwd_product_json', $_POST['dwd_product_json'][0] );
	}
	if ( isset( $_POST['dwd_product_file'] ) ) {
		update_post_meta( $post_id, 'dwd_product_file', $_POST['dwd_product_file'][0] );
	}
}

add_action( 'woocommerce_save_product_variation', function ( $variation_id, $loop ) {
	if ( empty( $_POST['dwd_product_uuid'][ $loop ] ) || $_POST['dwd_product_uuid'][ $loop ] === "0" ) {
		delete_post_meta( $variation_id, 'dwd_product_uuid' );
		delete_post_meta( $variation_id, 'dwd_product_json' );
		delete_post_meta( $variation_id, 'dwd_product_file' );
	} else {
		update_post_meta( $variation_id, 'dwd_product_uuid', $_POST['dwd_product_uuid'][ $loop ] );
		update_post_meta( $variation_id, 'dwd_product_json', $_POST['dwd_product_json'][ $loop ] );
		update_post_meta( $variation_id, 'dwd_product_file', $_POST['dwd_product_file'][ $loop ] );
	}
}, 10, 2 );

add_action( 'woocommerce_product_data_panels', function () {
	global $post;
	?>
    <div id="dwd_wc_view_product_tab" class='panel woocommerce_options_panel'>
		<?php dwd_wc_variable_selector( 0, null, $post ); ?>
    </div>
	<?php
} );

add_action( 'wp_ajax_dwd_wc_get_product_price', function () {
	// this function calculates the price of the shown selection
	// it does not incorporate the filtering for orderlines during
	// order injection
	$product    = $_POST['product'];
	$attributes = array_map( function ( $key ) use ( $product ) {
		return [ 'attribute' => $key, 'value' => $product['attributes'][ $key ] ];
	}, array_keys( $product['attributes'] ) );

	$api = DWD_WC_API_Helper::getInstance();

	// get minimal quantity
	try {
		$valid_quantities = $api->getQuantities( $product['uuid'], $product['attributes'] );
		$attributes[]     = [ 'attribute' => 'quantity', 'value' => min( $valid_quantities ) ];
	} catch ( Exception $e ) {
		$attributes[] = [ 'attribute' => 'quantity', 'value' => 1 ];
	}

	$price = $api->getPrice( $product['uuid'], $attributes );
	wp_send_json_success( [ 'price'         => $price['price'],
	                        'delivery_date' => $price['promisedArrivalDate']
	] );
} );


/** @noinspection PhpUnusedParameterInspection */
function dwd_wc_variable_selector( $loop, $variation_data, $variation ) {
	$products = DWD_WC_API_Helper::getInstance()->getProducts();
	/** @var $product array */
	$select_options = [ 0 => '' ];
	foreach ( $products as $product ) {
		$select_options[ $product['sku'] ] = $product['name'];
	}
	?>
    <div class="options_group" id="dwd-product-select-<?php echo $loop; ?>">
        <div>
            <div class='options_group'>
				<?php
				$product_uuid = get_post_meta( $variation->ID, 'dwd_product_uuid', true );

				woocommerce_wp_select( [
					'class'             => 'select short dwd_product_uuid',
					'id'                => "dwd_product_uuid{$loop}",
					'name'              => "dwd_product_uuid[{$loop}]",
					'options'           => $select_options,
					'label'             => __( 'Select the Drukwerkdeal product.', 'dwd-woocommerce' ),
					'wrapper_class'     => 'form-row form-row-full',
					'value'             => $product_uuid,
					'custom_attributes' => [ 'data-loop' => $loop, 'data-uuid' => $product_uuid ]
				] );
				?>
                <input
                        type="hidden"
                        name="dwd_product_json[<?php echo $loop; ?>]"
                        value="<?php echo esc_attr( get_post_meta( $variation->ID, 'dwd_product_json', true ) ); ?>"
                        id="dwd-json-<?php echo $loop; ?>"
                />
            </div>
        </div>
    </div>
	<?php if ( $product_uuid ): ?>
        <script type="text/javascript">
            initSelector("<?php echo $loop; ?>", "<?php echo $product_uuid; ?>", true);
        </script>
	<?php endif; ?>
    <div class="options_group">
        <div>
            <div class="options_group">
				<?php
				woocommerce_wp_text_input(
					array(
						'id'            => "dwd_product_file{$loop}",
						'name'          => "dwd_product_file[{$loop}]",
						'value'         => get_post_meta( $variation->ID, 'dwd_product_file', true ),
						'label'         => __( 'File to use for Drukwerkdeal orders (optional)', 'dwd-woocommerce' ),
						'placeholder'   => __( 'https://www.example.com/file.pdf', 'dwd-woocommerce' ),
						'description'   => __( 'Enter a url with a valid (supported) file here.', 'dwd-woocommerce' ),
						'type'          => 'text',
						'desc_tip'      => true,
						'wrapper_class' => 'form-row form-row-first',
					)
				);
				?>
            </div>
        </div>
    </div>
	<?php
}